@@ -148,6 +148,12 @@ class AssignmentList(LoggingConfigurable):
                     }
             else:
                 for assignment in assignments:
+                    for submission in assignment["submissions"]:
+                        if submission['local_feedback_path'] \
+                            and os.path.exists(submission['local_feedback_path']):
+                                submission['local_feedback_path'] = \
+                                    os.path.relpath(submission['local_feedback_path'],
+                                    self._root_dir)
                     assignment["submissions"] = sorted(
                         assignment["submissions"],
                         key=lambda x: x["timestamp"])
