@@ -12,7 +12,7 @@ var AssignmentUI = Backbone.View.extend({
 
     events: {},
 
-    initialize: function () {
+    initialize: function (options) {
         this.$modal = undefined;
         this.$modal_duedate = undefined;
         this.$modal_timezone = undefined;
@@ -34,6 +34,7 @@ var AssignmentUI = Backbone.View.extend({
         this.listenTo(this.model, "request", this.animateSaving);
         this.listenTo(this.model, "sync", this.closeModal);
 
+        this.is_lab = options.is_lab || false;
         this.render();
     },
 
@@ -96,14 +97,23 @@ var AssignmentUI = Backbone.View.extend({
 
     render: function () {
         this.clear();
-
+        var this_assignment = this;
         // assignment name
         var name = this.model.get("name")
         this.$name.attr("data-order", name);
+
+        /* Append link with :
+         *     - href if this is a Notebook<7 environment
+         *     - click listener to send message to iframe parent if this is Jupyter Lab environment
+         */
         this.$name.append($("<a/>")
-            .attr("target", "_blank")
-            .attr("href", base_url + "/tree/" + url_prefix + "/" + this.model.get("source_path"))
-            .text(name));
+            .text(name)
+            .attr("target", self.is_lab ? undefined : "_blank")
+            .attr("href", self.is_lab ? undefined : base_url + "/tree/" + url_prefix + "/" + this.model.get("source_path"))
+            .click(self.is_lab ? function(){
+                window.parent.postMessage(jlab_go_to_path(url_prefix + "/" + this_assignment.model.get("source_path")), '*');
+            } : undefined)
+        );
 
         // duedate
         var duedate = this.model.get("duedate");
@@ -144,9 +154,16 @@ var AssignmentUI = Backbone.View.extend({
         // preview student version
         var release_path = this.model.get("release_path");
         if (release_path) {
+            /* Append link with :
+             *     - href if this is a Notebook<7 environment
+             *     - click listener to send message to iframe parent if this is Jupyter Lab environment
+             */
             this.$preview.append($("<a/>")
-                .attr("target", "_blank")
-                .attr("href", base_url + "/tree/" + url_prefix + "/" + release_path)
+                .attr("target", self.is_lab ? undefined : "_blank")
+                .attr("href", self.is_lab ? undefined : base_url + "/tree/" + url_prefix + "/" + release_path)
+                .click(self.is_lab ? function(){
+                        window.parent.postMessage(jlab_go_to_path(url_prefix + "/" + release_path), '*');
+                    } : undefined)
                 .append($("<span/>")
                     .addClass("glyphicon glyphicon-search")
                     .attr("aria-hidden", "true")));
@@ -562,7 +579,7 @@ var createAssignmentModal = function () {
     modal = createModal("add-assignment-modal", "Add New Assignment", body, footer);
 };
 
-var loadAssignments = function () {
+var loadAssignments = function (is_lab=false) {
     var tbl = $("#main-table");
 
     models = new Assignments();
@@ -574,7 +591,8 @@ var loadAssignments = function () {
             models.each(function (model) {
                 var view = new AssignmentUI({
                     "model": model,
-                    "el": insertRow(tbl)
+                    "el": insertRow(tbl),
+                    "is_lab": is_lab
                 });
                 views.push(view);
             });
@@ -586,6 +604,9 @@ var loadAssignments = function () {
 
 var models = undefined;
 var views = [];
+
+var is_lab = is_lab || false;
+
 $(window).on('load', function () {
-    loadAssignments();
+    loadAssignments(is_lab);
 });
