@@ -1,6 +1,3 @@
-import json
-import sys
-
 from os.path import join
 
 from .. import run_nbgrader
@@ -25,6 +22,19 @@ class TestNbGraderValidate(BaseTestApp):
         output = run_nbgrader(["validate", "submitted-changed.ipynb"], stdout=True)
         assert output.strip() == "Success! Your notebook passes all the tests."
 
+    def test_validate_subdir(self):
+        """Does the validation fail on an unchanged notebook?"""
+        self._copy_file(join("files", "open_relative_file.ipynb"), "my_subdir/open_relative_file.ipynb")
+        self._copy_file(join("files", "data.txt"), "my_subdir/data.txt")
+        output = run_nbgrader(["validate", "my_subdir/open_relative_file.ipynb"], stdout=True)
+        assert output.strip() == "Success! Your notebook passes all the tests."
+
+    def test_validate_zero_points(self):
+      """Does validation correctly fail when cell has zero points?"""
+      self._copy_file(join("files", "validation-zero-points.ipynb"), "validation-zero-points.ipynb")
+      output = run_nbgrader(["validate", "validation-zero-points.ipynb"], stdout=True)
+      assert output.splitlines()[0] == "VALIDATION FAILED ON 1 CELL(S)! If you submit your assignment as it is, you WILL NOT"
+
     def test_invert_validate_unchanged(self):
         """Does the inverted validation pass on an unchanged notebook?"""
         self._copy_file(join("files", "submitted-unchanged.ipynb"), "submitted-unchanged.ipynb")
@@ -48,7 +58,7 @@ class TestNbGraderValidate(BaseTestApp):
         self._copy_file(join("files", "submitted-grade-cell-changed.ipynb"), "submitted-grade-cell-changed.ipynb")
         output = run_nbgrader([
             "validate", "submitted-grade-cell-changed.ipynb",
-            "--DisplayAutoGrades.ignore_checksums=True"
+            "--Validator.ignore_checksums=True"
         ], stdout=True)
         assert output.splitlines()[0] == "Success! Your notebook passes all the tests."
 
@@ -64,7 +74,7 @@ class TestNbGraderValidate(BaseTestApp):
         output = run_nbgrader([
             "validate", "submitted-grade-cell-changed.ipynb",
             "--invert",
-            "--DisplayAutoGrades.ignore_checksums=True"
+            "--Validator.ignore_checksums=True"
         ], stdout=True)
         assert output.splitlines()[0] == "NOTEBOOK PASSED ON 2 CELL(S)!"
 
@@ -73,7 +83,7 @@ class TestNbGraderValidate(BaseTestApp):
         self._copy_file(join("files", "submitted-unchanged.ipynb"), "submitted-unchanged.ipynb")
         output = run_nbgrader([
             "validate", "submitted-unchanged.ipynb",
-            "--DisplayAutoGrades.ignore_checksums=True"
+            "--Validator.ignore_checksums=True"
         ], stdout=True)
         assert output.splitlines()[0] == "VALIDATION FAILED ON 1 CELL(S)! If you submit your assignment as it is, you WILL NOT"
 
@@ -88,7 +98,7 @@ class TestNbGraderValidate(BaseTestApp):
         self._copy_file(join("files", "submitted-locked-cell-changed.ipynb"), "submitted-locked-cell-changed.ipynb")
         output = run_nbgrader([
             "validate", "submitted-locked-cell-changed.ipynb",
-            "--DisplayAutoGrades.ignore_checksums=True"
+            "--Validator.ignore_checksums=True"
         ], stdout=True)
         assert output.splitlines()[0] == "VALIDATION FAILED ON 1 CELL(S)! If you submit your assignment as it is, you WILL NOT"
 
@@ -104,23 +114,22 @@ class TestNbGraderValidate(BaseTestApp):
         output = run_nbgrader([
             "validate", "submitted-locked-cell-changed.ipynb",
             "--invert",
-            "--DisplayAutoGrades.ignore_checksums=True"
+            "--Validator.ignore_checksums=True"
         ], stdout=True)
         assert output.splitlines()[0] == "NOTEBOOK PASSED ON 1 CELL(S)!"
 
-    def test_invert_locked_cell_changed_ignore_checksums_json(self):
-        """Does the validate fail if a locked cell has changed with --invert and ignoring checksums?"""
-        self._copy_file(join("files", "submitted-locked-cell-changed.ipynb"), "submitted-locked-cell-changed.ipynb")
-        output = run_nbgrader([
-            "validate", "submitted-locked-cell-changed.ipynb",
-            "--invert",
-            "--json",
-            "--DisplayAutoGrades.ignore_checksums=True"
-        ], stdout=True)
-        assert json.loads(output) == {
-            "passed": [
-                {
-                    "source": 'print("Success!")'
-                }
-            ]
-        }
+    def test_validate_glob(self):
+        """Does the validation work when we glob filenames?"""
+        self._copy_file(join("files", "submitted-unchanged.ipynb"), "nb1.ipynb")
+        self._copy_file(join("files", "submitted-changed.ipynb"), "nb2.ipynb")
+        self._copy_file(join("files", "submitted-changed.ipynb"), "nb3.ipynb")
+        run_nbgrader(["validate", "*.ipynb"])
+        run_nbgrader(["validate", "nb1.ipynb", "nb2.ipynb"])
+        run_nbgrader(["validate", "nb1.ipynb", "nb2.ipynb", "nb3.ipynb"])
+        run_nbgrader(["validate"], retcode=1)
+
+    def test_validate_with_validating_envvar(self, db, course_dir):
+        self._copy_file(join("files", "validating-environment-variable.ipynb"), "nb1.ipynb")
+        output = run_nbgrader(["validate", "nb1.ipynb"], stdout=True)
+        assert output.splitlines()[0] == "VALIDATION FAILED ON 1 CELL(S)! If you submit your assignment as it is, you WILL NOT"
+
